import {IComponentOptions, ILogService} from "angular";
import {IModalService} from "angular-ui-bootstrap";
import {Promise} from "es6-promise";
import {NotificationService} from "../../shared/notification/notification.service";
import {IUser} from "../../shared/user/user.model";
import {UserService} from "../../shared/user/user.service";
import "./manage-users-page.less";

class ManageUsersPageController {

    private users: IUser[] = [];
    private loadingUsers = false;

    /* @ngInject */
    constructor(private $log: ILogService,
                private $uibModal: IModalService,
                private notificationService: NotificationService,
                private userService: UserService) {
        $log.info("AdminLoginPageController");

        this.handleDelete = this.handleDelete.bind(this);
    }

    public $onInit() {
        this.loadingUsers = true;
        this.userService.list()
            .then((users) => {
                this.users = users;
                this.loadingUsers = false;
            });
    }

    public inviteUser() {
        const modalInstance = this.$uibModal.open({
            animation: true,
            component: "userInviteModal"
        });

        modalInstance.result
            .then((invite) => {
                return this.userService.invite(invite)
                    .then(() => {
                        this.notificationService.success("Invite sent");
                    });
            }, () => {
                // ignore modal dismissal
            })
            .catch((error) => {
                this.notificationService.error(error.message);
            });
    }

    public handleDelete(users: IUser[]) {
        const promises = users.map((user) => this.userService.remove(user));

        Promise.all(promises)
            .then(() => {
                this.notificationService.success("Deleted");
            })
            .catch((error) => {
                this.notificationService.error(error.message);
            });
    }
}

export const ManageUsersPageComponent: IComponentOptions = {
    controller: ManageUsersPageController,
    template: require("./manage-users-page.html")
};

